@@ -1,9 +1,7 @@
-from __future__ import unicode_literals
-
-import hashlib
+import html
 import json
-import time
 import warnings
+from datetime import timedelta
 
 from django import forms
 from django.conf import settings
@@ -17,7 +15,6 @@ from django.contrib.auth import (
 from django.contrib.auth.models import AbstractUser
 from django.contrib.auth.password_validation import validate_password
 from django.contrib.sites.shortcuts import get_current_site
-from django.core.cache import cache
 from django.core.mail import EmailMessage, EmailMultiAlternatives
 from django.http import HttpResponse, HttpResponseRedirect
 from django.shortcuts import resolve_url
@@ -25,16 +22,21 @@ from django.template import TemplateDoesNotExist
 from django.template.loader import render_to_string
 from django.urls import reverse
 from django.utils import timezone
+from django.utils.crypto import get_random_string
 from django.utils.encoding import force_str
 from django.utils.translation import gettext_lazy as _
 
-from ..utils import (
+from allauth import ratelimit
+from allauth.account import signals
+from allauth.account.app_settings import EmailVerificationMethod
+from allauth.utils import (
     build_absolute_uri,
     email_address_exists,
     generate_unique_username,
     get_user_model,
     import_attribute,
 )
+
 from . import app_settings
 
 
@@ -96,7 +98,7 @@ class DefaultAccountAdapter(object):
         """
         return settings.DEFAULT_FROM_EMAIL
 
-    def render_mail(self, template_prefix, email, context):
+    def render_mail(self, template_prefix, email, context, headers=None):
         """
         Renders an e-mail to `email`.  `template_prefix` identifies the
         e-mail that is to be sent, e.g. "account/email/email_confirmation"
@@ -123,11 +125,13 @@ class DefaultAccountAdapter(object):
                     # We need at least one body
                     raise
         if "txt" in bodies:
-            msg = EmailMultiAlternatives(subject, bodies["txt"], from_email, to)
+            msg = EmailMultiAlternatives(
+                subject, bodies["txt"], from_email, to, headers=headers
+            )
             if "html" in bodies:
                 msg.attach_alternative(bodies["html"], "text/html")
         else:
-            msg = EmailMessage(subject, bodies["html"], from_email, to)
+            msg = EmailMessage(subject, bodies["html"], from_email, to, headers=headers)
             msg.content_subtype = "html"  # Main content is now text/html
         return msg
 
@@ -323,12 +327,13 @@ class DefaultAccountAdapter(object):
             try:
                 if message_context is None:
                     message_context = {}
-                message = render_to_string(
+                escaped_message = render_to_string(
                     message_template,
                     message_context,
                     self.request,
                 ).strip()
-                if message:
+                if escaped_message:
+                    message = html.unescape(escaped_message)
                     messages.add_message(request, level, message, extra_tags=extra_tags)
             except TemplateDoesNotExist:
                 pass
@@ -380,6 +385,67 @@ class DefaultAccountAdapter(object):
             form_spec["field_order"].append(field.html_name)
         return form_spec
 
+    def pre_login(
+        self,
+        request,
+        user,
+        *,
+        email_verification,
+        signal_kwargs,
+        email,
+        signup,
+        redirect_url
+    ):
+        from .utils import has_verified_email, send_email_confirmation
+
+        if not user.is_active:
+            return self.respond_user_inactive(request, user)
+
+        if email_verification == EmailVerificationMethod.NONE:
+            pass
+        elif email_verification == EmailVerificationMethod.OPTIONAL:
+            # In case of OPTIONAL verification: send on signup.
+            if not has_verified_email(user, email) and signup:
+                send_email_confirmation(request, user, signup=signup, email=email)
+        elif email_verification == EmailVerificationMethod.MANDATORY:
+            if not has_verified_email(user, email):
+                send_email_confirmation(request, user, signup=signup, email=email)
+                return self.respond_email_verification_sent(request, user)
+
+    def post_login(
+        self,
+        request,
+        user,
+        *,
+        email_verification,
+        signal_kwargs,
+        email,
+        signup,
+        redirect_url
+    ):
+        from .utils import get_login_redirect_url
+
+        response = HttpResponseRedirect(
+            get_login_redirect_url(request, redirect_url, signup=signup)
+        )
+
+        if signal_kwargs is None:
+            signal_kwargs = {}
+        signals.user_logged_in.send(
+            sender=user.__class__,
+            request=request,
+            response=response,
+            user=user,
+            **signal_kwargs,
+        )
+        self.add_message(
+            request,
+            messages.SUCCESS,
+            "account/messages/logged_in.txt",
+            {"user": user},
+        )
+        return response
+
     def login(self, request, user):
         # HACK: This is not nice. The proper Django way is to use an
         # authentication backend
@@ -448,6 +514,25 @@ class DefaultAccountAdapter(object):
         ret = build_absolute_uri(request, url)
         return ret
 
+    def should_send_confirmation_mail(self, request, email_address):
+        from allauth.account.models import EmailConfirmation
+
+        cooldown_period = timedelta(seconds=app_settings.EMAIL_CONFIRMATION_COOLDOWN)
+        if app_settings.EMAIL_CONFIRMATION_HMAC:
+            send_email = ratelimit.consume(
+                request,
+                action="confirm_email",
+                key=email_address.email,
+                amount=1,
+                duration=cooldown_period.total_seconds(),
+            )
+        else:
+            send_email = not EmailConfirmation.objects.filter(
+                sent__gt=timezone.now() - cooldown_period,
+                email_address=email_address,
+            ).exists()
+        return send_email
+
     def send_confirmation_mail(self, request, emailconfirmation, signup):
         current_site = get_current_site(request)
         activate_url = self.get_email_confirmation_url(request, emailconfirmation)
@@ -472,31 +557,26 @@ class DefaultAccountAdapter(object):
     def _get_login_attempts_cache_key(self, request, **credentials):
         site = get_current_site(request)
         login = credentials.get("email", credentials.get("username", "")).lower()
-        login_key = hashlib.sha256(login.encode("utf8")).hexdigest()
-        return "allauth/login_attempts@{site_id}:{login}".format(
-            site_id=site.pk, login=login_key
-        )
+        return "{site}:{login}".format(site=site.domain, login=login)
 
     def _delete_login_attempts_cached_email(self, request, **credentials):
         if app_settings.LOGIN_ATTEMPTS_LIMIT:
             cache_key = self._get_login_attempts_cache_key(request, **credentials)
-            cache.delete(cache_key)
+            ratelimit.clear(request, action="login_failed", key=cache_key)
 
     def pre_authenticate(self, request, **credentials):
         if app_settings.LOGIN_ATTEMPTS_LIMIT:
             cache_key = self._get_login_attempts_cache_key(request, **credentials)
-            login_data = cache.get(cache_key, None)
-            if login_data:
-                dt = timezone.now()
-                current_attempt_time = time.mktime(dt.timetuple())
-                if len(
-                    login_data
-                ) >= app_settings.LOGIN_ATTEMPTS_LIMIT and current_attempt_time < (
-                    login_data[-1] + app_settings.LOGIN_ATTEMPTS_TIMEOUT
-                ):
-                    raise forms.ValidationError(
-                        self.error_messages["too_many_login_attempts"]
-                    )
+            if not ratelimit.consume(
+                request,
+                action="login_failed",
+                key=cache_key,
+                amount=app_settings.LOGIN_ATTEMPTS_LIMIT,
+                duration=app_settings.LOGIN_ATTEMPTS_TIMEOUT,
+            ):
+                raise forms.ValidationError(
+                    self.error_messages["too_many_login_attempts"]
+                )
 
     def authenticate(self, request, **credentials):
         """Only authenticates, does not actually login. See `login`"""
@@ -514,12 +594,7 @@ class DefaultAccountAdapter(object):
         return user
 
     def authentication_failed(self, request, **credentials):
-        if app_settings.LOGIN_ATTEMPTS_LIMIT:
-            cache_key = self._get_login_attempts_cache_key(request, **credentials)
-            data = cache.get(cache_key, [])
-            dt = timezone.now()
-            data.append(time.mktime(dt.timetuple()))
-            cache.set(cache_key, data, app_settings.LOGIN_ATTEMPTS_TIMEOUT)
+        pass
 
     def is_ajax(self, request):
         return any(
@@ -530,6 +605,18 @@ class DefaultAccountAdapter(object):
             ]
         )
 
+    def get_client_ip(self, request):
+        x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
+        if x_forwarded_for:
+            ip = x_forwarded_for.split(",")[0]
+        else:
+            ip = request.META.get("REMOTE_ADDR")
+        return ip
+
+    def generate_emailconfirmation_key(self, email):
+        key = get_random_string(64).lower()
+        return key
+
 
 def get_adapter(request=None):
     return import_attribute(app_settings.ADAPTER)(request)
