@@ -13,8 +13,11 @@ from django.views.decorators.debug import sensitive_post_parameters
 from django.views.generic.base import TemplateResponseMixin, TemplateView, View
 from django.views.generic.edit import FormView
 
-from ..exceptions import ImmediateHttpResponse
-from ..utils import get_form_class, get_request_param
+from allauth import ratelimit
+from allauth.decorators import rate_limit
+from allauth.exceptions import ImmediateHttpResponse
+from allauth.utils import get_form_class, get_request_param
+
 from . import app_settings, signals
 from .adapter import get_adapter
 from .forms import (
@@ -35,12 +38,12 @@ from .utils import (
     logout_on_password_change,
     passthrough_next_redirect_url,
     perform_login,
+    send_email_confirmation,
     sync_user_email_addresses,
     url_str_to_user_pk,
 )
 
 
-INTERNAL_RESET_URL_KEY = "set-password"
 INTERNAL_RESET_SESSION_KEY = "_password_reset_key"
 
 
@@ -214,6 +217,7 @@ class CloseableSignupMixin(object):
         return self.response_class(**response_kwargs)
 
 
+@method_decorator(rate_limit(action="signup"), name="dispatch")
 class SignupView(
     RedirectAuthenticatedUserMixin,
     CloseableSignupMixin,
@@ -406,6 +410,7 @@ class ConfirmEmailView(TemplateResponseMixin, LogoutFunctionalityMixin, View):
 confirm_email = ConfirmEmailView.as_view()
 
 
+@method_decorator(rate_limit(action="manage_email"), name="dispatch")
 class EmailView(AjaxCapableProcessFormViewMixin, FormView):
     template_name = "account/email." + app_settings.TEMPLATE_EXTENSION
     form_class = AddEmailForm
@@ -450,7 +455,7 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
                 res = self._action_remove(request)
             elif "action_primary" in request.POST:
                 res = self._action_primary(request)
-            res = res or HttpResponseRedirect(self.success_url)
+            res = res or HttpResponseRedirect(self.get_success_url())
             # Given that we bypassed AjaxCapableProcessFormViewMixin,
             # we'll have to call invoke it manually...
             res = _ajax_response(request, res, data=self._get_ajax_data_if())
@@ -460,34 +465,29 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
             res = _ajax_response(request, res, data=self._get_ajax_data_if())
         return res
 
-    def _action_send(self, request, *args, **kwargs):
+    def _get_email_address(self, request):
         email = request.POST["email"]
         try:
-            email_address = EmailAddress.objects.get(
-                user=request.user,
-                email=email,
-            )
-            get_adapter(request).add_message(
-                request,
-                messages.INFO,
-                "account/messages/email_confirmation_sent.txt",
-                {"email": email},
-            )
-            email_address.send_confirmation(request)
-            return HttpResponseRedirect(self.get_success_url())
+            return EmailAddress.objects.get_for_user(user=request.user, email=email)
         except EmailAddress.DoesNotExist:
             pass
 
+    def _action_send(self, request, *args, **kwargs):
+        email_address = self._get_email_address(request)
+        if email_address:
+            send_email_confirmation(
+                self.request, request.user, email=email_address.email
+            )
+
     def _action_remove(self, request, *args, **kwargs):
-        email = request.POST["email"]
-        try:
-            email_address = EmailAddress.objects.get(user=request.user, email=email)
+        email_address = self._get_email_address(request)
+        if email_address:
             if email_address.primary:
                 get_adapter(request).add_message(
                     request,
                     messages.ERROR,
                     "account/messages/cannot_delete_primary_email.txt",
-                    {"email": email},
+                    {"email": email_address.email},
                 )
             else:
                 email_address.delete()
@@ -501,18 +501,13 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
                     request,
                     messages.SUCCESS,
                     "account/messages/email_deleted.txt",
-                    {"email": email},
+                    {"email": email_address.email},
                 )
                 return HttpResponseRedirect(self.get_success_url())
-        except EmailAddress.DoesNotExist:
-            pass
 
     def _action_primary(self, request, *args, **kwargs):
-        email = request.POST["email"]
-        try:
-            email_address = EmailAddress.objects.get_for_user(
-                user=request.user, email=email
-            )
+        email_address = self._get_email_address(request)
+        if email_address:
             # Not primary=True -- Slightly different variation, don't
             # require verified unless moving from a verified
             # address. Ignore constraint if previous primary email
@@ -551,8 +546,6 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
                     to_email_address=email_address,
                 )
                 return HttpResponseRedirect(self.get_success_url())
-        except EmailAddress.DoesNotExist:
-            pass
 
     def get_context_data(self, **kwargs):
         ret = super(EmailView, self).get_context_data(**kwargs)
@@ -579,6 +572,7 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
 email = login_required(EmailView.as_view())
 
 
+@method_decorator(rate_limit(action="change_password"), name="dispatch")
 class PasswordChangeView(AjaxCapableProcessFormViewMixin, FormView):
     template_name = "account/password_change." + app_settings.TEMPLATE_EXTENSION
     form_class = ChangePasswordForm
@@ -629,6 +623,12 @@ class PasswordChangeView(AjaxCapableProcessFormViewMixin, FormView):
 password_change = login_required(PasswordChangeView.as_view())
 
 
+@method_decorator(
+    # NOTE: 'change_password' (iso 'set_') is intentional, there is no need to
+    # differentiate between set and change.
+    rate_limit(action="change_password"),
+    name="dispatch",
+)
 class PasswordSetView(AjaxCapableProcessFormViewMixin, FormView):
     template_name = "account/password_set." + app_settings.TEMPLATE_EXTENSION
     form_class = SetPasswordForm
@@ -677,6 +677,7 @@ class PasswordSetView(AjaxCapableProcessFormViewMixin, FormView):
 password_set = login_required(PasswordSetView.as_view())
 
 
+@method_decorator(rate_limit(action="reset_password"), name="dispatch")
 class PasswordResetView(AjaxCapableProcessFormViewMixin, FormView):
     template_name = "account/password_reset." + app_settings.TEMPLATE_EXTENSION
     form_class = ResetPasswordForm
@@ -687,6 +688,13 @@ class PasswordResetView(AjaxCapableProcessFormViewMixin, FormView):
         return get_form_class(app_settings.FORMS, "reset_password", self.form_class)
 
     def form_valid(self, form):
+        r429 = ratelimit.consume_or_429(
+            self.request,
+            action="reset_password_email",
+            key=form.cleaned_data["email"],
+        )
+        if r429:
+            return r429
         form.save(self.request)
         return super(PasswordResetView, self).form_valid(form)
 
@@ -712,12 +720,14 @@ class PasswordResetDoneView(TemplateView):
 password_reset_done = PasswordResetDoneView.as_view()
 
 
+@method_decorator(rate_limit(action="reset_password_from_key"), name="dispatch")
 class PasswordResetFromKeyView(
     AjaxCapableProcessFormViewMixin, LogoutFunctionalityMixin, FormView
 ):
     template_name = "account/password_reset_from_key." + app_settings.TEMPLATE_EXTENSION
     form_class = ResetPasswordKeyForm
     success_url = reverse_lazy("account_reset_password_from_key_done")
+    reset_url_key = "set-password"
 
     def get_form_class(self):
         return get_form_class(
@@ -728,7 +738,7 @@ class PasswordResetFromKeyView(
         self.request = request
         self.key = key
 
-        if self.key == INTERNAL_RESET_URL_KEY:
+        if self.key == self.reset_url_key:
             self.key = self.request.session.get(INTERNAL_RESET_SESSION_KEY, "")
             # (Ab)using forms here to be able to handle errors in XHR #890
             token_form = UserTokenForm(data={"uidb36": uidb36, "key": self.key})
@@ -756,9 +766,7 @@ class PasswordResetFromKeyView(
                 # avoids the possibility of leaking the key in the
                 # HTTP Referer header.
                 self.request.session[INTERNAL_RESET_SESSION_KEY] = self.key
-                redirect_url = self.request.path.replace(
-                    self.key, INTERNAL_RESET_URL_KEY
-                )
+                redirect_url = self.request.path.replace(self.key, self.reset_url_key)
                 return redirect(redirect_url)
 
         self.reset_user = None
